@@ -150,8 +150,9 @@ DockApp::~DockApp(void)
 void
 DockApp::mapWindow(void)
 {
-    if (_mapped)
+    if (_mapped) {
         return;
+    }
     _mapped = true;
 
     XSelectInput(_dpy, _dockapp_window, NoEventMask);
@@ -165,8 +166,9 @@ DockApp::mapWindow(void)
 void
 DockApp::unmapWindow(void)
 {
-    if (!_mapped)
+    if (! _mapped) {
         return;
+    }
     _mapped = false;
 
     XSelectInput(_dpy, _dockapp_window, NoEventMask);
@@ -225,7 +227,7 @@ DockApp::repaint(void)
     pm->returnPixmap(_background);
     _background = pm->getPixmap(_gm.width, _gm.height, _scr->getDepth());
 
-    _theme->getHarbourTexture()->render(_background, 0, 0, _gm.width, _gm.height);
+    _theme->getHarbourData()->getTexture()->render(_background, 0, 0, _gm.width, _gm.height);
 
     XSetWindowBackgroundPixmap(_dpy, _window, _background);
     XClearWindow(_dpy, _window);
@@ -278,8 +280,8 @@ DockApp::readClassHint(void)
 {
     XClassHint x_class_hint;
     if (XGetClassHint(_dpy, _client_window, &x_class_hint)) {
-        _class_hint.h_name = x_class_hint.res_name;
-        _class_hint.h_class = x_class_hint.res_class;
+        _class_hint.h_name = Util::to_wide_str(x_class_hint.res_name);
+        _class_hint.h_class = Util::to_wide_str(x_class_hint.res_class);
         XFree(x_class_hint.res_name);
         XFree(x_class_hint.res_class);
     }
@@ -291,7 +293,7 @@ DockApp::readAutoProperties(void)
 {
     DockAppProperty *prop =
         AutoProperties::instance()->findDockAppProperty(&_class_hint);
-    if (prop != NULL) {
+    if (prop) {
         _position = prop->getPosition();
     }
 }
