@@ -1,12 +1,15 @@
 //
 // Harbour.cc for pekwm
-// Copyright (C) 2003-2004 Claes Nasten <pekdon{@}pekdon{.}net>
+// Copyright © 2003-2004 Claes Nästen <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
+
 #ifdef HARBOUR
 
 #include "Harbour.hh"
@@ -39,16 +42,14 @@ using std::endl;
 Harbour::Harbour(PScreen *s, Theme *t, Workspaces *w) :
         _scr(s), _theme(t), _workspaces(w),
 #ifdef MENUS
-        _harbour_menu(NULL),
+        _harbour_menu(0),
 #endif // MENUS
-        _hidden(false), _size(0), _strut(NULL),
+        _hidden(false), _size(0), _strut(0),
         _last_button_x(0), _last_button_y(0)
 {
     _strut = new Strut();
     _scr->addStrut(_strut);
-#ifdef HAVE_XINERAMA
     _strut->head = Config::instance()->getHarbourHead();
-#endif // HAVE_XINERAMA
 
 #ifdef MENUS
     _harbour_menu = new HarbourMenu(_scr, _theme, this);
@@ -64,7 +65,7 @@ Harbour::~Harbour(void)
     delete _strut;
 
 #ifdef MENUS
-    if (_harbour_menu != NULL) {
+    if (_harbour_menu) {
         delete _harbour_menu;
     }
 #endif // MENUS
@@ -75,7 +76,7 @@ Harbour::~Harbour(void)
 void
 Harbour::addDockApp(DockApp *da)
 {
-    if (da == NULL) {
+    if (! da) {
         return;
     }
 
@@ -91,7 +92,7 @@ Harbour::addDockApp(DockApp *da)
     da->setLayer(Config::instance()->isHarbourOntop() ? LAYER_DOCK : LAYER_DESKTOP);
     _workspaces->insert(da); // add the dockapp to the stacking list
 
-    if (da->isMapped() == false) { // make sure it's visible
+    if (! da->isMapped()) { // make sure it's visible
         da->mapWindow();
     }
 
@@ -102,7 +103,7 @@ Harbour::addDockApp(DockApp *da)
 void
 Harbour::removeDockApp(DockApp *da)
 {
-    if (da == NULL)
+    if (! da)
         return;
 
     list<DockApp*>::iterator it(find(_da_list.begin(), _da_list.end(), da));
@@ -136,7 +137,7 @@ Harbour::removeAllDockApps(void)
 DockApp*
 Harbour::findDockApp(Window win)
 {
-    DockApp *dockapp = NULL;
+    DockApp *dockapp = 0;
 
     list<DockApp*>::iterator it(_da_list.begin());
     for (; it != _da_list.end(); ++it) {
@@ -153,7 +154,7 @@ Harbour::findDockApp(Window win)
 DockApp*
 Harbour::findDockAppFromFrame(Window win)
 {
-    DockApp *dockapp = NULL;
+    DockApp *dockapp = 0;
 
     list<DockApp*>::iterator it(_da_list.begin());
     for (; it != _da_list.end(); ++it) {
@@ -182,12 +183,10 @@ Harbour::updateGeometry(void)
 void
 Harbour::restack(void)
 {
-#ifdef HAVE_XINERAMA
     PScreen::instance()->removeStrut(_strut);
     if (Config::instance()->isHarbourOntop()) {
         PScreen::instance()->addStrut(_strut);
     }
-#endif // HAVE_XINERAMA
     uint l = Config::instance()->isHarbourOntop() ? LAYER_DOCK : LAYER_DESKTOP;
 
     list<DockApp*>::iterator it(_da_list.begin());
@@ -206,9 +205,7 @@ Harbour::restack(void)
 void
 Harbour::rearrange(void)
 {
-#ifdef HAVE_XINERAMA
     _strut->head = Config::instance()->getHarbourHead();
-#endif // HAVE_XINERAMA
 
     if (AutoProperties::instance()->isHarbourSort ())
         placeDockAppsSorted();
@@ -285,7 +282,7 @@ Harbour::updateStrutSize(void)
 {
     _strut->left = _strut->right = _strut->top = _strut->bottom = 0;
 
-    if (!Config::instance()->isHarbourMaximizeOver()) {
+    if (! Config::instance()->isHarbourMaximizeOver()) {
         switch (Config::instance()->getHarbourPlacement()) {
         case TOP:
             _strut->top = _size;
@@ -309,9 +306,10 @@ Harbour::updateStrutSize(void)
 void
 Harbour::handleButtonEvent(XButtonEvent* ev, DockApp* da)
 {
-    if (da == 0)
+    if (! da) {
         return;
-
+    }
+    
     _last_button_x = ev->x;
     _last_button_y = ev->y;
 
@@ -336,17 +334,14 @@ Harbour::handleButtonEvent(XButtonEvent* ev, DockApp* da)
 void
 Harbour::handleMotionNotifyEvent(XMotionEvent* ev, DockApp* da)
 {
-    if (da == NULL)
+    if (! da) {
         return;
-
+    }
+    
     Geometry head;
     int x = 0, y = 0;
 
-#ifdef HAVE_XINERAMA
     _scr->getHeadInfo(Config::instance()->getHarbourHead(), head);
-#else // !HAVE_XINERAMA
-    _scr->getHeadInfo(0, head);
-#endif // HAVE_XINERAMA
 
     switch(Config::instance()->getHarbourPlacement()) {
     case TOP:
@@ -381,9 +376,10 @@ Harbour::handleMotionNotifyEvent(XMotionEvent* ev, DockApp* da)
 void
 Harbour::handleConfigureRequestEvent(XConfigureRequestEvent* ev, DockApp* da)
 {
-    if (da == NULL)
+    if (! da) {
         return;
-
+    }
+    
     list<DockApp*>::iterator it(find(_da_list.begin(), _da_list.end(), da));
 
     if (it != _da_list.end()) {
@@ -404,7 +400,7 @@ Harbour::handleConfigureRequestEvent(XConfigureRequestEvent* ev, DockApp* da)
 void
 Harbour::placeDockApp(DockApp *da)
 {
-    if (!da || !_da_list.size ())
+    if (! da || ! _da_list.size ())
         return;
 
     bool right = (Config::instance()->getHarbourOrientation() == BOTTOM_TO_TOP);
@@ -413,27 +409,22 @@ Harbour::placeDockApp(DockApp *da)
     bool placed = false, increase = false, x_place = false;
 
     Geometry head;
-#ifdef HAVE_XINERAMA
     PScreen::instance()->getHeadInfo(Config::instance()->getHarbourHead(), head);
-#else // !HAVE_XINERAMA
-    PScreen::instance()->getHeadInfo(0, head);
-#endif // HAVE_XINERAMA
 
     getPlaceStartPosition (da, x, y, x_place);
-    if (right)
-    {
-        if (x_place)
+    if (right) {
+        if (x_place) {
             x -= da->getWidth ();
-        else
+        } else {
             y -= da->getHeight ();
+        }
     }
 
     list<DockApp*>::iterator it;
-    if (x_place)
-    {
+    if (x_place) {
         x = test = right ? head.x + head.width - da->getWidth() : head.x;
 
-        while (!placed
+        while (! placed
                 && (right
                     ? (test >= 0)
                     : ((test + da->getWidth()) < (head.x + head.width))))
@@ -441,30 +432,28 @@ Harbour::placeDockApp(DockApp *da)
             placed = increase = true;
 
             it = _da_list.begin();
-            for (; placed && (it != _da_list.end()); ++it)
-            {
-                if ((*it) == da)
+            for (; placed && (it != _da_list.end()); ++it) {
+                if ((*it) == da) {
                     continue; // exclude ourselves
-
-                if (((*it)->getX() < signed(test + da->getWidth())) &&
-                        ((*it)->getRX() > test))
-                {
+                }
+                
+                if (((*it)->getX() < static_cast<signed>(test + da->getWidth())) &&
+                        ((*it)->getRX() > test)) {
                     placed = increase = false;
                     test = right ? (*it)->getX() - da->getWidth() : (*it)->getRX();
                 }
             }
 
-            if (placed)
+            if (placed) {
                 x = test;
-            else if (increase)
+            } else if (increase) {
                 test += right ? -1 : 1;
+            }
         }
-    }
-    else // !x_place
-    {
+    } else { // !x_place
         y = test = right ? head.y + head.height - da->getHeight() : head.y;
 
-        while (!placed
+        while (! placed
                 && (right
                     ? (test >= 0)
                     : ((test + da->getHeight()) < (head.y + head.height))))
@@ -472,23 +461,23 @@ Harbour::placeDockApp(DockApp *da)
             placed = increase = true;
 
             it = _da_list.begin();
-            for (; placed && (it != _da_list.end()); ++it)
-            {
-                if ((*it) == da)
+            for (; placed && (it != _da_list.end()); ++it) {
+                if ((*it) == da) {
                     continue; // exclude ourselves
-
-                if (((*it)->getY() < signed(test + da->getHeight())) &&
-                        ((*it)->getBY() > test))
-                {
+                }
+                
+                if (((*it)->getY() < static_cast<signed>(test + da->getHeight())) &&
+                        ((*it)->getBY() > test)) {
                     placed = increase = false;
                     test = right ? (*it)->getY() - da->getHeight() : (*it)->getBY();
                 }
             }
 
-            if (placed)
+            if (placed) {
                 y = test;
-            else if (increase)
+            } else if (increase) {
                 test += right ? -1 : 1;
+            }
         }
     }
 
@@ -501,9 +490,10 @@ Harbour::placeDockApp(DockApp *da)
 void
 Harbour::placeDockAppsSorted(void)
 {
-    if (!_da_list.size ())
+    if (! _da_list.size ()) {
         return;
-
+    }
+    
     // place the dockapps
     int x, y, x_real, y_real;
     bool inc_x = false;
@@ -512,32 +502,22 @@ Harbour::placeDockAppsSorted(void)
     getPlaceStartPosition (_da_list.front (), x_real, y_real, inc_x);
 
     list<DockApp*>::iterator it (_da_list.begin ());
-    for (; it != _da_list.end (); ++it)
-    {
+    for (; it != _da_list.end (); ++it) {
         getPlaceStartPosition (*it, x, y, inc_x);
 
-        if (inc_x)
-        {
-            if (right)
-            {
+        if (inc_x) {
+            if (right) {
                 (*it)->move (x_real - (*it)->getWidth (), y);
                 x_real -= -(*it)->getWidth ();
-            }
-            else
-            {
+            } else {
                 (*it)->move (x_real, y);
                 x_real += (*it)->getWidth ();
             }
-        }
-        else
-        {
-            if (right)
-            {
+        } else {
+            if (right) {
                 (*it)->move (x, y_real - (*it)->getHeight ());
                 y_real -= (*it)->getHeight ();
-            }
-            else
-            {
+            } else {
                 (*it)->move (x, y_real);
                 y_real += (*it)->getHeight ();
             }
@@ -545,17 +525,15 @@ Harbour::placeDockAppsSorted(void)
     }
 }
 
-//! @brief
+/**
+ * Make sure dock app is inside screen boundaries and placed on the
+ * right edge, usually called after resizing the dockapp.
+ */
 void
 Harbour::placeDockAppInsideScreen(DockApp *da)
 {
     Geometry head;
-#ifdef HAVE_XINERAMA
     PScreen::instance()->getHeadInfo(Config::instance()->getHarbourHead(), head);
-#else // !HAVE_XINEAMA
-    PScreen::instance()->getHeadInfo(0, head);
-#endif // HAVE_XINERAMA
-
     uint pos = Config::instance()->getHarbourPlacement();
 
     // top or bottom placement
@@ -563,14 +541,14 @@ Harbour::placeDockAppInsideScreen(DockApp *da)
         // check horizontal position
         if (da->getX() < head.x) {
             da->move(head.x, da->getY());
-        } else if (da->getRX() > signed(head.x + head.width)) {
+        } else if (da->getRX() > static_cast<signed>(head.x + head.width)) {
             da->move(head.x + head.width - da->getWidth(), da->getY());
         }
 
         // check vertical position
-        if ((pos == TOP) && (da->getY() < head.y)) {
+        if ((pos == TOP) && (da->getY() != head.y)) {
             da->move(da->getX(), head.y);
-        } else if ((pos == BOTTOM) && (da->getBY() > signed(head.y + head.height))) {
+        } else if ((pos == BOTTOM) && (da->getBY() != static_cast<signed>(head.y + head.height))) {
             da->move(da->getX(), head.y + head.height - da->getHeight());
         }
 
@@ -579,14 +557,14 @@ Harbour::placeDockAppInsideScreen(DockApp *da)
         // check vertical position
         if (da->getY() < head.y) {
             da->move(da->getX(), head.y);
-        } else if (da->getBY() > signed(head.y + head.height)) {
+        } else if (da->getBY() > static_cast<signed>(head.y + head.height)) {
             da->move(da->getX(), head.y + head.height - da->getHeight());
         }
 
         // check horizontal position
-        if ((pos == LEFT) && (da->getX() < head.x)) {
+        if ((pos == LEFT) && (da->getX() != head.x)) {
             da->move(head.x, da->getY());
-        } else if ((pos == RIGHT) && (da->getRX() > signed(head.x + head.width))) {
+        } else if ((pos == RIGHT) && (da->getRX() != static_cast<signed>(head.x + head.width))) {
             da->move(head.x + head.width - da->getWidth(), da->getY());
         }
     }
@@ -596,16 +574,12 @@ Harbour::placeDockAppInsideScreen(DockApp *da)
 void
 Harbour::getPlaceStartPosition(DockApp *da, int &x, int &y, bool &inc_x)
 {
-    if (!da)
+    if (! da) {
         return;
-
+    }
+    
     Geometry head;
-#ifdef HAVE_XINERAMA
     PScreen::instance()->getHeadInfo(Config::instance()->getHarbourHead(), head);
-#else // !HAVE_XINEAMA
-    PScreen::instance()->getHeadInfo(0, head);
-#endif // HAVE_XINERAMA
-
     bool right = (Config::instance()->getHarbourOrientation() == BOTTOM_TO_TOP);
 
     switch (Config::instance()->getHarbourPlacement()) {
@@ -643,17 +617,14 @@ Harbour::insertDockAppSorted(DockApp *da)
     // anyway, order goes as follows: 1 2 3 0 0 0 -3 -2 -1
 
     // Middle of the list.
-    if (da->getPosition () == 0)
-    {
+    if (da->getPosition () == 0) {
         for (; (it != _da_list.end ()) && ((*it)->getPosition () >= 0); ++it)
             ;
         // Beginning of the list.
     } else if (da->getPosition () > 0) {
-        for (; it != _da_list.end (); ++it)
-        {
+        for (; it != _da_list.end (); ++it) {
             if (((*it)->getPosition () < 1) || // got to 0 or less
-                    (da->getPosition () <= (*it)->getPosition ()))
-            {
+                    (da->getPosition () <= (*it)->getPosition ())) {
                 break;
             }
         }
