@@ -1,12 +1,14 @@
 //
 // KeyGrabber.cc for pekwm
-// Copyright (C) 2003-2005 Claes Nasten <pekdon{@}pekdon{.}net>
+// Copyright © 2003-2008 Claes Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
 
 #include "KeyGrabber.hh"
 
@@ -29,7 +31,7 @@ using std::list;
 using std::vector;
 using std::find;
 
-KeyGrabber* KeyGrabber::_instance = NULL;
+KeyGrabber* KeyGrabber::_instance = 0;
 
 //! @brief Constructor for Chain class
 KeyGrabber::Chain::Chain(uint mod, uint key) :
@@ -68,7 +70,7 @@ KeyGrabber::Chain::findChain(XKeyEvent *ev)
         }
     }
 
-    return NULL;
+    return 0;
 }
 
 //! @brief Searches the _keys list for an action
@@ -84,23 +86,23 @@ KeyGrabber::Chain::findAction(XKeyEvent *ev)
         }
     }
 
-    return NULL;
+    return 0;
 }
 
 //! @brief KeyGrabber constructor
-KeyGrabber::KeyGrabber(PScreen *scr) :
-        _scr(scr),
+KeyGrabber::KeyGrabber(PScreen *scr)
+    : _scr(scr),
 #ifdef MENUS
-        _menu_chain(0, 0),
+      _menu_chain(0, 0),
 #endif // MENUS
-        _global_chain(0, 0), _moveresize_chain(0, 0),
-        _cmd_d_chain(0, 0)
+      _global_chain(0, 0), _moveresize_chain(0, 0),
+      _input_dialog_chain(0, 0)
 {
 #ifdef DEBUG
-    if (_instance != NULL) {
+    if (_instance) {
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "KeyGrabber(" << this << ")::KeyGrabber(" << scr << ")" << endl
-             << " *** _instance allready set: " << _instance << endl;
+             << " *** _instance already set: " << _instance << endl;
     }
 #endif // DEBUG
     _instance = this;
@@ -112,87 +114,90 @@ KeyGrabber::KeyGrabber(PScreen *scr) :
 //! @brief Destructor for KeyGrabber class
 KeyGrabber::~KeyGrabber(void)
 {
-    _instance = NULL;
+    _instance = 0;
 }
 
 //! @brief Parses the "KeyFile" and inserts into _global_keys.
 //! If _global_keys holds any keygrabs they will be flushed before
 //! reloading the new keybindings.
-void
+bool
 KeyGrabber::load(const std::string &file)
 {
-    CfgParser key_cfg;
+    if (! Util::requireReload(_cfg_state, file)) {
+        return false;
+    }
 
-    if (!key_cfg.parse (file, CfgParserSource::SOURCE_FILE))
-    {
-        if (!key_cfg.parse (SYSCONFDIR "/keys", CfgParserSource::SOURCE_FILE))
-        {
-            cerr << __FILE__ << "@" << __LINE__
-                 << "Error: no keyfile at " << file << " or "
-                 << SYSCONFDIR << "/keys" << endl;
-            return;
+    CfgParser key_cfg;
+    if (! key_cfg.parse(file, CfgParserSource::SOURCE_FILE)) {
+        _cfg_state.clear();
+        if (! key_cfg.parse(SYSCONFDIR "/keys", CfgParserSource::SOURCE_FILE, true)) {
+            cerr << __FILE__ << "@" << __LINE__ << "Error: no keyfile at " << file
+                 << " or " << SYSCONFDIR "/keys" << endl;
+            return false;
         }
     }
 
-    CfgParser::Entry *op_section;
-
-    op_section = key_cfg.get_entry_root ()->find_section ("GLOBAL");
-    if (op_section)
-    {
-        _global_chain.unload ();
-        parseGlobalChain (op_section, &_global_chain);
+    if (key_cfg.is_dynamic_content()) {
+        _cfg_state.clear();
+    } else {
+        _cfg_state = key_cfg.get_file_list();
     }
 
-    op_section = key_cfg.get_entry_root ()->find_section ("MOVERESIZE");
-    if (op_section)
-    {
+    CfgParser::Entry *section;
+
+    section = key_cfg.get_entry_root()->find_section("GLOBAL");
+    if (section) {
+        _global_chain.unload();
+        parseGlobalChain(section, &_global_chain);
+    }
+
+    section = key_cfg.get_entry_root ()->find_section("MOVERESIZE");
+    if (section) {
         _moveresize_chain.unload ();
-        parseMoveResizeChain (op_section, &_moveresize_chain);
+        parseMoveResizeChain (section, &_moveresize_chain);
     }
 
-    op_section = key_cfg.get_entry_root ()->find_section ("CMDDIALOG");
-    if (op_section)
-    {
-        _cmd_d_chain.unload ();
-        parseCmdDialogChain (op_section, &_cmd_d_chain);
+    // Previously there was only a CmdDialog section, however the text
+    // handling parts have been moved into InputDialog but to keep
+    // compatibility this check exists.
+    section = key_cfg.get_entry_root()->find_section("INPUTDIALOG");
+    if (! section) {
+        section = key_cfg.get_entry_root()->find_section("CMDDIALOG");
+    }
+
+    if (section) {
+        _input_dialog_chain.unload ();
+        parseInputDialogChain (section, &_input_dialog_chain);
     }
 
 #ifdef MENUS
-    op_section = key_cfg.get_entry_root ()->find_section ("MENU");
-    if (op_section)
-    {
-        _menu_chain.unload ();
-        parseMenuChain (op_section, &_menu_chain);
+    section = key_cfg.get_entry_root ()->find_section("MENU");
+    if (section) {
+        _menu_chain.unload();
+        parseMenuChain(section, &_menu_chain);
     }
 #endif // MENUS
+
+    return true;
 }
 
 //! @brief Parses chain, getting actions as plain ActionEvents
 void
-KeyGrabber::parseGlobalChain(CfgParser::Entry *op_section,
-                             KeyGrabber::Chain *chain)
+KeyGrabber::parseGlobalChain(CfgParser::Entry *section, KeyGrabber::Chain *chain)
 {
-    op_section = op_section->get_section ();
-
     ActionEvent ae;
     uint key, mod;
 
-    while ((op_section = op_section->get_section_next ()) != NULL)
-    {
-        if (*op_section == "CHAIN")
-        {
+    CfgParser::iterator it(section->begin());
+    for (; it != section->end(); ++it) {
+        if ((*it)->get_section() && *(*it) == "CHAIN") {
             // Figure out mod and key, create a new chain.
-            if (Config::instance ()->parseKey (op_section->get_value (),
-                                               mod, key))
-            {
-                KeyGrabber::Chain *sub_chain = new KeyGrabber::Chain (mod, key);
-                parseGlobalChain (op_section, sub_chain);
-                chain->addChain (sub_chain);
+            if (Config::instance()->parseKey((*it)->get_value(), mod, key)) {
+                KeyGrabber::Chain *sub_chain = new KeyGrabber::Chain(mod, key);
+                parseGlobalChain((*it)->get_section(), sub_chain);
+                chain->addChain(sub_chain);
             }
-        }
-        else if (Config::instance ()->parseActionEvent(op_section, ae,
-                 KEYGRABBER_OK, false))
-        {
+        } else if (Config::instance()->parseActionEvent((*it), ae, KEYGRABBER_OK, false)) {
             chain->addAction(ae);
         }
     }
@@ -200,59 +205,44 @@ KeyGrabber::parseGlobalChain(CfgParser::Entry *op_section,
 
 //! @brief Parses chain, getting actions as MoveResizeEvents
 void
-KeyGrabber::parseMoveResizeChain(CfgParser::Entry *op_section,
-                                 KeyGrabber::Chain *chain)
+KeyGrabber::parseMoveResizeChain(CfgParser::Entry *section, KeyGrabber::Chain *chain)
 {
-    op_section = op_section->get_section ();
-
     ActionEvent ae;
     uint key, mod;
 
-    while ((op_section = op_section->get_section_next ()) != NULL)
-    {
-        if (*op_section == "CHAIN")
-        {
+    CfgParser::iterator it(section->begin());
+    for (; it != section->end(); ++it) {
+        if ((*it)->get_section() && *(*it) == "CHAIN") {
             // Figure out mod and key, create a new chain.
-            if (Config::instance ()->parseKey(op_section->get_value (), mod, key))
-            {
+            if (Config::instance()->parseKey((*it)->get_value(), mod, key)) {
                 KeyGrabber::Chain *sub_chain = new KeyGrabber::Chain(mod, key);
-                parseMoveResizeChain (op_section, sub_chain);
-                chain->addChain (sub_chain);
+                parseMoveResizeChain((*it)->get_section(), sub_chain);
+                chain->addChain(sub_chain);
             }
-        }
-        else if (Config::instance ()->parseMoveResizeEvent (op_section, ae))
-        {
-            chain->addAction (ae);
+        } else if (Config::instance()->parseMoveResizeEvent((*it), ae)) {
+            chain->addAction(ae);
         }
     }
 }
 
-//! @brief Parses chain, getting actions as CmdDialog Events
+//! @brief Parses chain, getting actions as InputDialog Events
 void
-KeyGrabber::parseCmdDialogChain(CfgParser::Entry *op_section,
-                                KeyGrabber::Chain *chain)
+KeyGrabber::parseInputDialogChain(CfgParser::Entry *section, KeyGrabber::Chain *chain)
 {
-    op_section = op_section->get_section ();
-
     ActionEvent ae;
     uint key, mod;
 
-    while ((op_section = op_section->get_section_next ()) != NULL)
-    {
-        if (*op_section == "CHAIN")
-        {
+    CfgParser::iterator it(section->begin());
+    for (; it != section->end(); ++it) {
+        if ((*it)->get_section() && *(*it) == "CHAIN") {
             // Figure out mod and key, create a new chain.
-            if (Config::instance ()->parseKey (op_section->get_value (),
-                                               mod, key))
-            {
+            if (Config::instance()->parseKey((*it)->get_value(), mod, key)) {
                 KeyGrabber::Chain *sub_chain = new KeyGrabber::Chain(mod, key);
-                parseCmdDialogChain (op_section, sub_chain);
-                chain->addChain (sub_chain);
+                parseInputDialogChain((*it)->get_section(), sub_chain);
+                chain->addChain(sub_chain);
             }
-        }
-        else if (Config::instance()->parseCmdDialogEvent (op_section, ae))
-        {
-            chain->addAction (ae);
+        } else if (Config::instance()->parseInputDialogEvent((*it), ae)) {
+            chain->addAction(ae);
         }
     }
 }
@@ -260,29 +250,22 @@ KeyGrabber::parseCmdDialogChain(CfgParser::Entry *op_section,
 #ifdef MENUS
 //! @brief Parses chain, getting actions as MenuEvents
 void
-KeyGrabber::parseMenuChain(CfgParser::Entry *op_section,
-                           KeyGrabber::Chain *chain)
+KeyGrabber::parseMenuChain(CfgParser::Entry *section, KeyGrabber::Chain *chain)
 {
-    op_section = op_section->get_section ();
-
     ActionEvent ae;
     uint key, mod;
 
-    while ((op_section = op_section->get_section_next ()) != NULL)
-    {
-        if (*op_section == "CHAIN")
-        {
+    CfgParser::iterator it(section->begin());
+    for (; it != section->end(); ++it) {
+        if ((*it)->get_section() && *(*it) == "CHAIN") {
             // Figure out mod and key, create a new chain.
-            if (Config::instance ()->parseKey(op_section->get_value (), mod, key))
-            {
+            if (Config::instance()->parseKey((*it)->get_value(), mod, key)) {
                 KeyGrabber::Chain *sub_chain = new KeyGrabber::Chain (mod, key);
-                parseGlobalChain (op_section, sub_chain);
-                chain->addChain (sub_chain);
+                parseGlobalChain((*it)->get_section(), sub_chain);
+                chain->addChain(sub_chain);
             }
-        }
-        else if (Config::instance ()->parseMenuEvent (op_section, ae))
-        {
-            chain->addAction (ae);
+        } else if (Config::instance()->parseMenuEvent((*it), ae)) {
+            chain->addAction(ae);
         }
     }
 }
@@ -349,19 +332,19 @@ KeyGrabber::ungrabKeys(Window win)
 ActionEvent*
 KeyGrabber::findAction(XKeyEvent *ev, KeyGrabber::Chain *chain)
 {
-    if (ev == NULL) {
-        return NULL;
+    if (! ev) {
+        return 0;
     }
     ev->state &= ~_num_lock & ~_scroll_lock & ~LockMask;
 
-    ActionEvent *action = NULL;
+    ActionEvent *action = 0;
     KeyGrabber::Chain *sub_chain = _global_chain.findChain(ev);
     if (sub_chain && _scr->grabKeyboard(_scr->getRoot())) {
         XEvent c_ev;
         KeyGrabber::Chain *last_chain;
         bool exit = false;
 
-        while (!exit) {
+        while (! exit) {
             XMaskEvent(_scr->getDpy(), KeyPressMask, &c_ev);
             c_ev.xkey.state &= ~_num_lock & ~_scroll_lock & ~LockMask;
 
@@ -388,19 +371,19 @@ KeyGrabber::findAction(XKeyEvent *ev, KeyGrabber::Chain *chain)
 ActionEvent*
 KeyGrabber::findAction(XKeyEvent *ev, PWinObj::Type type)
 {
-    ActionEvent *ae = NULL;
+    ActionEvent *ae = 0;
 
 #ifdef MENUS
     if (type == PWinObj::WO_MENU) {
         ae = findAction(ev, &_menu_chain);
     }
 #endif // MENUS
-    if (type == PWinObj::WO_CMD_DIALOG) {
-        ae = findAction(ev, &_cmd_d_chain);
+    if (type == PWinObj::WO_CMD_DIALOG || type == PWinObj::WO_SEARCH_DIALOG) {
+        ae = findAction(ev, &_input_dialog_chain);
     }
 
     // no action the menu list, try the global list
-    if (ae == NULL) {
+    if (! ae) {
         ae = findAction(ev, &_global_chain);
     }
 
