@@ -1,12 +1,14 @@
 //
 // PMenu.cc for pekwm
-// Copyright (C) 2004-2006 Claes Nasten <pekdon{@}pekdon{.}net>
+// Copyright © 2004-2008 Claes Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
 
 #include "PWinObj.hh"
 #include "PDecor.hh"
@@ -18,9 +20,11 @@
 #include "ActionHandler.hh"
 #include "Config.hh"
 #include "ScreenResources.hh"
+#include "TextureHandler.hh"
 #include "Theme.hh"
 #include "PixmapHandler.hh"
 #include "Workspaces.hh"
+#include "AutoProperties.hh"
 
 #include <algorithm>
 #include <cstdlib>
@@ -34,18 +38,38 @@ using std::endl;
 using std::list;
 using std::map;
 using std::string;
+using std::wstring;
 using std::find;
 
+PMenu::Item::Item(const std::wstring &name, PWinObj *wo_ref, PTexture *icon)
+  : _x(0), _y(0), _name(name), _wo_ref(wo_ref),
+    _type(MENU_ITEM_NORMAL), _icon(icon), _creator(0)
+{
+    if (_icon) {
+        TextureHandler::instance()->referenceTexture(_icon);
+    }
+}
+
+PMenu::Item::~Item(void)
+{
+    if (_icon) {
+        TextureHandler::instance()->returnTexture(_icon);
+    }
+}
+
 map<Window,PMenu*> PMenu::_menu_map = map<Window,PMenu*>();
 
 //! @brief Constructor for PMenu class
-PMenu::PMenu(Display *dpy, Theme *theme, const std::string &title,
+PMenu::PMenu(Display *dpy, Theme *theme, const std::wstring &title,
              const std::string &name, const std::string decor_name)
     : PDecor(dpy, theme, decor_name),
       _name(name),
-      _menu_wo(NULL), _menu_parent(NULL),
+      _menu_parent(0), _class_hint(L"pekwm", L"Menu", L"", L"", L""),
+      _menu_wo(0),
       _menu_bg_fo(None), _menu_bg_un(None), _menu_bg_se(None),
+      _menu_width(0),
       _item_height(0), _item_width_max(0), _item_width_max_avail(0),
+      _icon_width(0), _icon_height(0),
       _separator_height(0),
       _rows(0), _cols(0), _scroll(false), _has_submenu(false)
 {
@@ -57,10 +81,6 @@ PMenu::PMenu(Display *dpy, Theme *theme, const std::string &title,
     _layer = LAYER_MENU;
     _hidden = true; // don't care about it when changing worskpace etc
 
-    titleAdd(&_title);
-    titleSetActive(0);
-    setTitle(title);
-
     // create menu content child
     _menu_wo = new PWinObj(_dpy);
     XSetWindowAttributes attr;
@@ -71,6 +91,11 @@ PMenu::PMenu(Display *dpy, Theme *theme, const std::string &title,
                                       0, 0, 1, 1, 0,
                                       CopyFromParent, InputOutput, CopyFromParent,
                                       CWOverrideRedirect|CWEventMask, &attr));
+
+    titleAdd(&_title);
+    titleSetActive(0);
+    setTitle(title);
+	
     addChild(_menu_wo);
     addChildWindow(_menu_wo->getWindow());
     activateChild(_menu_wo);
@@ -91,7 +116,7 @@ PMenu::~PMenu(void)
     Workspaces::instance()->remove(this);
 
     // Free resources
-    if (_menu_wo != NULL) {
+    if (_menu_wo) {
         _child_list.remove(_menu_wo);
         removeChildWindow(_menu_wo->getWindow());
         XDestroyWindow(_dpy, _menu_wo->getWindow());
@@ -137,6 +162,16 @@ PMenu::setFocused(bool focused)
     }
 }
 
+/**
+ * Set focusable, includes the _menu_wo as well as the decor.
+ */
+void
+PMenu::setFocusable(bool focusable)
+{
+    PDecor::setFocusable(focusable);
+    _menu_wo->setFocusable(focusable);
+}
+
 //! @brief
 ActionEvent*
 PMenu::handleButtonPress(XButtonEvent *ev)
@@ -156,10 +191,18 @@ PMenu::handleButtonPress(XButtonEvent *ev)
     }
 }
 
-//! @brief
+/**
+ * Handle button release.
+ */
 ActionEvent*
 PMenu::handleButtonRelease(XButtonEvent *ev)
 {
+    if (_window == ev->subwindow) {
+        ev->window = _menu_wo->getWindow();
+        ev->x -= _gm.x;
+        ev->y -= _gm.y + getTitleHeight();
+    }
+
     if (*_menu_wo == ev->window) {
         MouseEventType mb = MOUSE_EVENT_RELEASE;
 
@@ -185,24 +228,34 @@ PMenu::handleButtonRelease(XButtonEvent *ev)
     }
 }
 
-//! @brief
+/**
+ * Handle motion event, select buttons and execute actions.
+ *
+ * @param ev Handle motion event.
+ */
 ActionEvent*
 PMenu::handleMotionEvent(XMotionEvent *ev)
 {
+    if (_window == ev->subwindow) {
+        ev->window = _menu_wo->getWindow();
+        ev->x -= _gm.x;
+        ev->y -= _gm.y + getTitleHeight();
+    }
+
     if (*_menu_wo == ev->window) {
-        handleItemEvent(MOUSE_EVENT_MOTION, ev->x, ev->y);
+        uint button = PScreen::instance()->getButtonFromState(ev->state);
+        handleItemEvent(button ? MOUSE_EVENT_MOTION_PRESSED : MOUSE_EVENT_MOTION, ev->x, ev->y);
 
         ActionEvent *ae;
-        uint button = PScreen::instance()->getButtonFromState(ev->state);
-        ae = ActionHandler::findMouseAction(button, ev->state,
-                                            MOUSE_EVENT_MOTION,
+        ev->state = PScreen::instance()->stripButtonModifiers(ev->state);
+        ae = ActionHandler::findMouseAction(button, ev->state, MOUSE_EVENT_MOTION,
                                             Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_MENU));
 
         // check motion threshold
-        if ((ae != NULL) && (ae->threshold > 0)) {
-            if (!ActionHandler::checkAEThreshold(ev->x_root, ev->y_root,
-                                                 _pointer_x, _pointer_y, ae->threshold)) {
-                ae = NULL;
+        if (ae && (ae->threshold > 0)) {
+            if (! ActionHandler::checkAEThreshold(ev->x_root, ev->y_root,
+                                                  _pointer_x, _pointer_y, ae->threshold)) {
+                ae = 0;
             }
         }
         return ae;
@@ -216,8 +269,7 @@ ActionEvent*
 PMenu::handleEnterEvent(XCrossingEvent *ev)
 {
     if (*_menu_wo == ev->window) {
-        return ActionHandler::findMouseAction(BUTTON_ANY, ev->state,
-                                              MOUSE_EVENT_ENTER,
+        return ActionHandler::findMouseAction(BUTTON_ANY, ev->state, MOUSE_EVENT_ENTER,
                                               Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_MENU));
     } else {
         return PDecor::handleEnterEvent(ev);
@@ -239,29 +291,47 @@ PMenu::handleLeaveEvent(XCrossingEvent *ev)
 
 // END - PWinObj interface.
 
-//! @brief
+// START - PDecor interface.
+
+/**
+ * Load menu theme after border has been updated.
+ */
+void
+PMenu::loadTheme(void)
+{
+    buildMenuRender();
+}
+
+// END - PDecor interface.
+
+/**
+ * Handle event on menu item at position, ignores event if no item
+ * exists at the position.
+ */
 void
 PMenu::handleItemEvent(MouseEventType type, int x, int y)
 {
     PMenu::Item *item = findItem(x, y);
-    if (!item)
+    if (! item) {
         return;
+    }
 
     // Unmap submenu if we enter them on the same event as selecting.
     if (((_item_curr == _item_list.end()) || (item != *_item_curr))
-            && Config::instance()->isMenuSelectOn(type))
+        && Config::instance()->isMenuSelectOn(type)) {
         select(item, Config::instance()->isMenuEnterOn(type));
+    }
 
     if (Config::instance()->isMenuEnterOn(type)) {
-        if ((item->getWORef() != NULL) &&
-                (item->getWORef()->getType() == PWinObj::WO_MENU)) {
-            // special case for motion, would flicker like crazy if we didn't check
-            if ((type != MOUSE_EVENT_MOTION) &&
-                    (item->getWORef()->isMapped() == true)) {
+        if (item->getWORef()
+            && (item->getWORef()->getType() == PWinObj::WO_MENU)) {
+            // Special case for motion, would flicker like crazy if we didn't check
+            if ((type != MOUSE_EVENT_MOTION) && (type != MOUSE_EVENT_MOTION_PRESSED)
+                && item->getWORef()->isMapped()) {
                 static_cast<PMenu*>(item->getWORef())->unmapSubmenus();
                 item->getWORef()->unmapWindow();
 
-            } else if (item->getWORef()->isMapped() == false) {
+            } else if (! item->getWORef()->isMapped()) {
                 // unmap previous opened submenu if any
                 unmapSubmenus();
                 mapSubmenu(static_cast<PMenu*>(item->getWORef()));
@@ -270,8 +340,7 @@ PMenu::handleItemEvent(MouseEventType type, int x, int y)
     }
 
     // Submenus don't have any actions, so we don't exec ( and close them )
-    if (item->getAE().action_list.size()
-            && Config::instance()->isMenuExecOn(type)) {
+    if (item->getAE().action_list.size() && Config::instance()->isMenuExecOn(type)) {
         exec(item);
     }
 }
@@ -297,13 +366,12 @@ PMenu::buildMenu(void)
 void
 PMenu::buildMenuCalculate(void)
 {
-    uint width = 1, height = 1, sep = 0;
-    list<PMenu::Item*>::iterator it;
     _has_submenu = false;
 
-    // get how many visible objects we have
-    _size = 0;
-    for (it = _item_list.begin(); it != _item_list.end(); ++it) {
+    // Get how many visible objects we have
+    unsigned int sep = 0;
+    list<PMenu::Item*>::iterator it(_item_list.begin());
+    for (_size = 0; it != _item_list.end(); ++it) {
         if ((*it)->getType() == PMenu::Item::MENU_ITEM_NORMAL) {
             ++_size;
         } else if ((*it)->getType() == PMenu::Item::MENU_ITEM_SEPARATOR) {
@@ -312,31 +380,15 @@ PMenu::buildMenuCalculate(void)
     }
 
     if (_size == 0) {
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": "
-             << "PMenu(" << this << ")::buildMenuCalculate()" << endl
-             << " *** _size == 0" << endl;
-#endif // DEBUG
         return;
     }
 
-    // calculate max item width, to be used if/when splitting a menu up
-    // in rows because of limited vertical space.
-    _item_width_max = 1;
-    for (it = _item_list.begin(); it != _item_list.end(); ++it) {
-        // don't include dynamic etc
-        if ((*it)->getType() == PMenu::Item::MENU_ITEM_NORMAL) {
-            // check if we have a submenu item
-            if (!_has_submenu && ((*it)->getWORef() != NULL) &&
-                    ((*it)->getWORef()->getType() == PWinObj::WO_MENU)) {
-                _has_submenu = true;
-            }
+    unsigned int width = 1, height = 1;
+    buildMenuCalculateMaxWidth(width, height);
 
-            width = _theme->getMenuData()->getFont(OBJECT_STATE_FOCUSED)->getWidth((*it)->getName().c_str());
-            if (width > _item_width_max) {
-                _item_width_max = width;
-            }
-        }
+    // FIXME: Remove extra padding from calculation
+    if (_menu_width) {
+      _item_width_max = _menu_width;
     }
 
     // This is the available width for drawing text on, the rest is reserved
@@ -345,36 +397,115 @@ PMenu::buildMenuCalculate(void)
 
     // Continue add padding etc.
     _item_width_max += _theme->getMenuData()->getPad(PAD_LEFT)
-                       + _theme->getMenuData()->getPad(PAD_RIGHT);
+        + _theme->getMenuData()->getPad(PAD_RIGHT);
+    if (Config::instance()->isDisplayMenuIcons()) {
+        _item_width_max += _icon_width;
+    }
 
     // If we have any submenus, increase the maximum width with arrow width +
     // right pad as we are going to pad the arrow from the text too.
     if (_has_submenu) {
         _item_width_max += _theme->getMenuData()->getPad(PAD_RIGHT)
-                           + _theme->getMenuData()->getTextureArrow(OBJECT_STATE_FOCUSED)->getWidth();
+            + _theme->getMenuData()->getTextureArrow(OBJECT_STATE_FOCUSED)->getWidth();
     }
 
-    // calculate item height
-    _item_height = _theme->getMenuData()->getFont(OBJECT_STATE_FOCUSED)->getHeight()  +
-                   _theme->getMenuData()->getPad(PAD_UP) +
-                   _theme->getMenuData()->getPad(PAD_DOWN);
+    // Remove padding etc from avail and item width.
+    if (_menu_width) {
+        unsigned int padding = _item_width_max - _item_width_max_avail;
+        _item_width_max -= padding;
+        _item_width_max_avail -= padding;
+    }
+
+    // Calculate item height
+    _item_height = std::max(_theme->getMenuData()->getFont(OBJECT_STATE_FOCUSED)->getHeight(), _icon_height)
+      + _theme->getMenuData()->getPad(PAD_UP)
+      + _theme->getMenuData()->getPad(PAD_DOWN);
     _separator_height = _theme->getMenuData()->getTextureSeparator(OBJECT_STATE_FOCUSED)->getHeight();
 
     height = (_item_height * _size) + (_separator_height * sep);
+    buildMenuCalculateColumns(width, height);
 
-    // check if menu should have more than one row
-    if ((height + getTitleHeight()) > PScreen::instance()->getHeight()) {
-        _cols = height / (PScreen::instance()->getHeight() - getTitleHeight());
-        if ((height % (PScreen::instance()->getHeight() - getTitleHeight())) != 0) {
-            ++_cols;
+    // Check if we need to enable scrolling
+    _scroll = (width > PScreen::instance()->getWidth());
+
+    resizeChild(width, height);
+}
+
+/**
+ * Get maximum item width and icon size.
+ */
+void
+PMenu::buildMenuCalculateMaxWidth(unsigned int &width, unsigned int &height)
+{
+    // Calculate max item width, to be used if/when splitting a menu
+    // up in rows because of limited vertical space.
+    _item_width_max = 1;
+    _icon_width = _icon_height = 0;
+
+    list<PMenu::Item*>::iterator it(_item_list.begin());
+    for (it = _item_list.begin(); it != _item_list.end(); ++it) {
+        // Only include standard items
+        if ((*it)->getType() != PMenu::Item::MENU_ITEM_NORMAL) {
+            continue;
         }
-        _rows = _size / _cols;
-        if ((_size % _cols) != 0) {
-            ++_rows;
+
+        // Check if we have a submenu item
+        if (! _has_submenu && (*it)->getWORef() &&
+            ((*it)->getWORef()->getType() == PWinObj::WO_MENU)) {
+          _has_submenu = true;
         }
-    } else {
+
+        // Get icon height if any
+        if ((*it)->getIcon()) {
+          if ((*it)->getIcon()->getWidth() > _icon_width) {
+            _icon_width = (*it)->getIcon()->getWidth();
+          }
+          if ((*it)->getIcon()->getHeight() > _icon_height) {
+            _icon_height = (*it)->getIcon()->getHeight();
+          }
+        }
+
+        width = _theme->getMenuData()->getFont(OBJECT_STATE_FOCUSED)->getWidth((*it)->getName().c_str());
+        if (width > _item_width_max) {
+            _item_width_max = width;
+        }
+    }
+
+
+    // Make sure icon width and height are not larger than configured.
+    if (_icon_width) {
+        _icon_width = Util::between<uint>(_icon_width,
+                                          Config::instance()->getMenuIconLimit(_icon_width, WIDTH_MIN, _name),
+                                          Config::instance()->getMenuIconLimit(_icon_width, WIDTH_MAX, _name));
+        _icon_height = Util::between<uint>(_icon_height,
+                                           Config::instance()->getMenuIconLimit(_icon_height, HEIGHT_MIN, _name),
+                                           Config::instance()->getMenuIconLimit(_icon_height, HEIGHT_MAX, _name));
+    }
+}
+
+/**
+ * Calculate number of columns, this does not apply to static width
+ * menus.
+ */
+void
+PMenu::buildMenuCalculateColumns(unsigned int &width, unsigned int &height)
+{
+    // Check if the menu fits or is static width
+    if (_menu_width
+        || (height + getTitleHeight()) <= PScreen::instance()->getHeight()) {
         _cols = 1;
+        width = _menu_width ? _menu_width : _item_width_max;
         _rows = _size;
+        return;
+    }
+
+    _cols = height / (PScreen::instance()->getHeight() - getTitleHeight());
+    if ((height % (PScreen::instance()->getHeight() - getTitleHeight())) != 0) {
+        ++_cols;
+    }
+    _rows = _size / _cols;
+    if ((_size % _cols) != 0) {
+        ++_rows;
     }
 
     width = _cols * _item_width_max;
@@ -382,7 +513,8 @@ PMenu::buildMenuCalculate(void)
     if (_cols > 1) {
         uint i, j, row_height;
         height = 0;
-
+       
+        list<PMenu::Item*>::iterator it(_item_list.begin());
         for (i = 0, it = _item_list.begin(); i < _cols; ++i) {
             row_height = 0;
             for (j = 0; (j < _rows) && (it != _item_list.end()); ++it) {
@@ -399,15 +531,6 @@ PMenu::buildMenuCalculate(void)
             }
         }
     }
-
-    // check if we need to enable scrolling
-    if (width > PScreen::instance()->getWidth()) {
-        _scroll = true;
-    } else {
-        _scroll = false;
-    }
-
-    resizeChild(width, height);
 }
 
 //! @brief Places the items in the menu
@@ -484,41 +607,60 @@ void
 PMenu::buildMenuRenderItem(Pixmap pix, ObjectState state, PMenu::Item *item)
 {
     PTexture *tex;
+    Theme::PMenuData *md = _theme->getMenuData();
 
     if (item->getType() == PMenu::Item::MENU_ITEM_NORMAL) {
-        tex = _theme->getMenuData()->getTextureItem(state);
-        tex->render(pix,
-                    item->getX(), item->getY(), _item_width_max, _item_height);
+        tex = md->getTextureItem(state);
+        tex->render(pix, item->getX(), item->getY(), _item_width_max, _item_height);
 
-        // If we have a submenu, lets draw our submenu "arrow"
+        uint start_x, start_y, icon_width, icon_height;
+        // If entry has an icon, draw it
+        if (item->getIcon() && Config::instance()->isDisplayMenuIcons()) {
+            icon_width = Util::between<uint>(item->getIcon()->getWidth(),
+                                             Config::instance()->getMenuIconLimit(_icon_width, WIDTH_MIN, _name),
+                                             Config::instance()->getMenuIconLimit(_icon_width, WIDTH_MAX, _name));
+            
+            icon_height = Util::between<uint>(item->getIcon()->getHeight(),
+                                              Config::instance()->getMenuIconLimit(_icon_height, HEIGHT_MIN, _name),
+                                              Config::instance()->getMenuIconLimit(_icon_height, HEIGHT_MAX, _name));
 
-        if (item->getWORef()
-                && (item->getWORef()->getType() == PWinObj::WO_MENU)) {
-            tex = _theme->getMenuData()->getTextureArrow(state);
-            uint a_w = tex->getWidth();
-            uint a_h = tex->getHeight();
-            uint a_y = static_cast<uint>((_item_height / 2) - (a_h / 2));
-
-            tex->render(pix,
-                        item->getX() + _item_width_max - a_w -
-                        _theme->getMenuData()->getPad(PAD_RIGHT),
-                        item->getY() + a_y,
-                        a_w, a_h);
+            start_x = item->getX() + md->getPad(PAD_LEFT) + (_icon_width - icon_width) / 2;
+            start_y = item->getY() + (_item_height - icon_height) / 2;
+            item->getIcon()->render(pix, start_x, start_y, icon_width, icon_height);
+        } else {
+            icon_width = 0;
+            icon_height = 0;
         }
 
-        PFont *font = _theme->getMenuData()->getFont(state);
-        font->draw(pix,
-                   item->getX() + _theme->getMenuData()->getPad(PAD_LEFT),
-                   item->getY() + _theme->getMenuData()->getPad(PAD_UP),
-                   item->getName().c_str(), 0, // Text and max chars
-                   _item_width_max_avail);
+        // If entry has a submenu, lets draw our submenu "arrow"
+        if (item->getWORef() && (item->getWORef()->getType() == PWinObj::WO_MENU)) {
+            tex = md->getTextureArrow(state);
+            uint arrow_width = tex->getWidth();
+            uint arrow_height = tex->getHeight();
+            uint arrow_y = static_cast<uint>((_item_height / 2) - (arrow_height / 2));
+
+            start_x = item->getX() + _item_width_max - arrow_width - md->getPad(PAD_RIGHT);
+            start_y = item->getY() + arrow_y;
+            tex->render(pix, start_x, start_y, arrow_width, arrow_height);
+        }
+
+        PFont *font = md->getFont(state);
+        start_x = item->getX() + md->getPad(PAD_LEFT);
+        // Add icon width to starting x position if frame icons are enabled.
+        if (Config::instance()->isDisplayMenuIcons()) {
+            start_x += _icon_width;
+        }
+		
+        start_y = item->getY() + md->getPad(PAD_UP)
+            + (_item_height - font->getHeight() - md->getPad(PAD_UP) - md->getPad(PAD_DOWN)) / 2;
+
+        // Render item text.
+        font->draw(pix, start_x, start_y, item->getName().c_str(), 0, _item_width_max_avail);
 
     } else if ((item->getType() == PMenu::Item::MENU_ITEM_SEPARATOR) &&
                (state < OBJECT_STATE_SELECTED)) {
-        tex = _theme->getMenuData()->getTextureSeparator(state);
-        tex->render(pix,
-                    item->getX(), item->getY(),
-                    _item_width_max, _separator_height);
+        tex = md->getTextureSeparator(state);
+        tex->render(pix, item->getX(), item->getY(), _item_width_max, _separator_height);
     }
 }
 
@@ -533,19 +675,20 @@ PMenu::buildMenuRenderItem(Pixmap pix, ObjectState state, PMenu::Item *item)
 void
 PMenu::selectItem(std::list<PMenu::Item*>::iterator item, bool unmap_submenu)
 {
-    if (_item_curr == item)
+    if (_item_curr == item) {
         return;
+    }
 
     deselectItem(unmap_submenu);
-
     _item_curr = item;
 
-    if (_mapped)
+    if (_mapped) {
         COPY_ITEM_AREA((*item), _menu_bg_se);
+    }
 }
 
 //! @brief Deselects selected item
-//! @aram unmap_submenu Defaults to true
+//! @param unmap_submenu Defaults to true
 void
 PMenu::deselectItem(bool unmap_submenu)
 {
@@ -626,9 +769,29 @@ PMenu::selectPrevItem(void)
 
 //! @brief Sets title of the menu/decor
 void
-PMenu::setTitle(const std::string &title)
+PMenu::setTitle(const std::wstring &title)
 {
     _title.setReal(title);
+	
+    // Apply title rules to allow title rewriting
+    applyTitleRules(title);
+}
+
+/**
+ * Applies title rules to menu.
+ */
+void
+PMenu::applyTitleRules(const std::wstring &title)
+{
+    _class_hint.title = title;
+    TitleProperty *data = AutoProperties::instance()->findTitleProperty(&_class_hint);
+
+    if (data) {
+        wstring new_title(title);
+        if (data->getTitleRule().ed_s(new_title)) {
+            _title.setCustom(new_title);
+        }
+    }
 }
 
 //! @brief Inserts item into the menu ( without rebuilding )
@@ -641,13 +804,13 @@ PMenu::insert(PMenu::Item *item)
 
 //! @brief Creates and inserts Item
 //! @param name Name of objet to create and insert
-//! @param wo_ref PWinObj to refer to, defaults to NULL
+//! @param wo_ref PWinObj to refer to, defaults to 0
 void
-PMenu::insert(const std::string &name, PWinObj *wo_ref)
+PMenu::insert(const std::wstring &name, PWinObj *wo_ref, PTexture *icon)
 {
     PMenu::Item *item;
 
-    item = new PMenu::Item(name, wo_ref);
+    item = new PMenu::Item(name, wo_ref, icon);
 
     insert(item);
 }
@@ -655,13 +818,13 @@ PMenu::insert(const std::string &name, PWinObj *wo_ref)
 //! @brief Creates and inserts Item
 //! @param name Name of object to create and insert
 //! @param ae ActionEvent for the object
-//! @param wo_ref PWinObj to refer to, defaults to NULL
+//! @param wo_ref PWinObj to refer to, defaults to 0
 void
-PMenu::insert(const std::string &name, const ActionEvent &ae, PWinObj *wo_ref)
+PMenu::insert(const std::wstring &name, const ActionEvent &ae, PWinObj *wo_ref, PTexture *icon)
 {
     PMenu::Item *item;
 
-    item = new PMenu::Item(name, wo_ref);
+    item = new PMenu::Item(name, wo_ref, icon);
     item->setAE(ae);
 
     insert(item);
@@ -671,17 +834,18 @@ PMenu::insert(const std::string &name, const ActionEvent &ae, PWinObj *wo_ref)
 void
 PMenu::remove(PMenu::Item *item)
 {
-    if (item == NULL) {
+    if (! item) {
 #ifdef DEBUG
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "PMenu(" << this << ")::remove(" << item << ")" << endl
-             << " *** item == NULL" << endl;
+             << " *** item == 0" << endl;
 #endif // DEBUG
         return;
     }
 
-    if ((_item_curr != _item_list.end()) && (item == *_item_curr))
+    if ((_item_curr != _item_list.end()) && (item == *_item_curr)) {
         _item_curr = _item_list.end();
+    }
 
     delete item;
     _item_list.remove(item);
@@ -696,6 +860,7 @@ PMenu::removeAll(void)
         delete *it;
     }
     _item_list.clear();
+    _item_curr = _item_list.end();
 }
 
 //! @brief Places the menu under the mouse and maps it.
@@ -736,7 +901,7 @@ PMenu::mapSubmenu(PMenu *menu, bool focus)
     menu->mapWindowRaised();
     menu->makeInsideScreen(x, y);
 
-    if (focus == true) {
+    if (focus) {
         menu->giveInputFocus();
         menu->selectItemNum(0);
     }
@@ -749,11 +914,9 @@ PMenu::unmapSubmenus(void)
     list<PMenu::Item*>::iterator it(_item_list.begin());
     for (; it != _item_list.end(); ++it) {
         if ((*it)->getWORef() && (*it)->getWORef()->getType() == PWinObj::WO_MENU) {
-            if (! (*it)->isDynamic()) {
-                // Sub-menus will be deleted when unmapping this, so no need
-                // to continue.
-                static_cast<PMenu*>((*it)->getWORef())->unmapSubmenus();
-            }
+            // Sub-menus will be deleted when unmapping this, so no need
+            // to continue.
+            static_cast<PMenu*>((*it)->getWORef())->unmapSubmenus();
             (*it)->getWORef()->unmapWindow();
         }
     }
@@ -763,7 +926,7 @@ PMenu::unmapSubmenus(void)
 void
 PMenu::unmapAll(void)
 {
-    if (_menu_parent != NULL) {
+    if (_menu_parent) {
         _menu_parent->unmapAll();
     } else {
         unmapSubmenus();
@@ -775,7 +938,7 @@ PMenu::unmapAll(void)
 void
 PMenu::gotoParentMenu(void)
 {
-    if (_menu_parent == NULL) {
+    if (! _menu_parent) {
         return;
     }
 
@@ -783,9 +946,9 @@ PMenu::gotoParentMenu(void)
     _menu_parent->giveInputFocus();
 }
 
-//! @brief Selects item, if NULL/not in list current item is deselected
+//! @brief Selects item, if 0/not in list current item is deselected
 //! @param item Item to select
-//! @aram unmap_submenu Defaults to true
+//! @param unmap_submenu Defaults to true
 void
 PMenu::select(PMenu::Item *item, bool unmap_submenu)
 {
@@ -853,11 +1016,11 @@ PMenu::selectItemRel(int off)
 void
 PMenu::exec(PMenu::Item *item)
 {
-    if (_menu_parent != NULL) {
+    if (_menu_parent) {
         _menu_parent->exec(item);
     } else {
         handleItemExec(item);
-        if (_sticky == false) {
+        if (! _sticky) {
             unmapAll();
         }
     }
@@ -867,7 +1030,7 @@ PMenu::exec(PMenu::Item *item)
 void
 PMenu::checkItemWORef(PMenu::Item *item)
 {
-    if ((item->getWORef() != NULL) &&
+    if (item->getWORef() &&
             (item->getWORef()->getType() == PWinObj::WO_MENU)) {
         PMenu *child = static_cast<PMenu*>(item->getWORef());
         child->_menu_parent = this;
@@ -886,7 +1049,7 @@ PMenu::findItem(int x, int y)
             return *it;
         }
     }
-    return NULL;
+    return 0;
 }
 
 //! @brief Moves the menu relative to it's parent to make it fit on screen
@@ -906,7 +1069,7 @@ PMenu::makeInsideScreen(int x, int y)
     if (x < head.x) {
         x = head.x;
     } else if ((x + _gm.width) > (head.x + head.width)) {
-        if (_menu_parent != NULL) {
+        if (_menu_parent) {
             x = _menu_parent->_gm.x - _gm.width; // not using getX(), refers to child
         } else {
             x = head.x + head.width - _gm.width;
