@@ -1,34 +1,35 @@
 //
 // PWinObj.cc for pekwm
-// Copyright (C)  2003-2006 Claes Nästen <me{@}pekdon{.}net>
+// Copyright © 2003-2007 Claes Nästen <me{@}pekdon{.}net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
 
+#ifdef HAVE_CONFIG_H
 #include "../config.h"
-#include "PWinObj.hh"
+#endif // HAVE_CONFIG_H
 
 #include <algorithm>
-#ifdef DEBUG
 #include <iostream>
+
+#include "PWinObj.hh"
+
 using std::cerr;
 using std::endl;
-#endif // DEBUG
-
 using std::find;
 using std::vector;
 using std::map;
 
-PWinObj* PWinObj::_focused_wo = (PWinObj*) NULL;
-PWinObj* PWinObj::_root_wo = (PWinObj*) NULL;
+PWinObj* PWinObj::_focused_wo = (PWinObj*) 0;
+PWinObj* PWinObj::_root_wo = (PWinObj*) 0;
 vector<PWinObj*> PWinObj::_wo_list = vector<PWinObj*>();
 map<Window, PWinObj*> PWinObj::_wo_map = map<Window, PWinObj*>();
 
 //! @brief PWinObj constructor.
 PWinObj::PWinObj(Display *dpy)
     : _dpy(dpy), _window(None),
-      _parent(NULL), _type(WO_NO_TYPE),
+      _parent(0), _type(WO_NO_TYPE),
       _workspace(0), _layer(LAYER_NORMAL),
       _mapped(false), _iconified(false), _hidden(false),
       _focused(false), _sticky(false),
@@ -40,7 +41,7 @@ PWinObj::PWinObj(Display *dpy)
 PWinObj::~PWinObj(void)
 {
     if (_focused_wo == this) {
-        _focused_wo = NULL;
+        _focused_wo = 0;
     }
 }
 
@@ -65,7 +66,7 @@ PWinObj::removeChildWindow(Window win)
 void
 PWinObj::mapWindow(void)
 {
-    if (_mapped == true) {
+    if (_mapped) {
         return;
     }
     _mapped = true;
@@ -78,7 +79,7 @@ PWinObj::mapWindow(void)
 void
 PWinObj::mapWindowRaised(void)
 {
-    if (_mapped == true) {
+    if (_mapped) {
         return;
     }
     _mapped = true;
@@ -91,8 +92,10 @@ PWinObj::mapWindowRaised(void)
 void
 PWinObj::unmapWindow(void)
 {
-    if (!_mapped)
+    if (! _mapped) {
         return;
+    }
+    
     _mapped = false;
 
     // Make sure unmapped windows drops focus
@@ -105,8 +108,10 @@ PWinObj::unmapWindow(void)
 void
 PWinObj::iconify(void)
 {
-    if (_iconified)
+    if (_iconified) {
         return;
+    }
+    
     _iconified = true;
 }
 
@@ -120,9 +125,8 @@ PWinObj::stick(void)
 //! @brief Moves the window and updates _gm.
 //! @param x X position
 //! @param y Y position
-//! @param do_virtual Update virtual postion, defaults to true
 void
-PWinObj::move(int x, int y, bool do_virtual)
+PWinObj::move(int x, int y)
 {
     _gm.x = x;
     _gm.y = y;
@@ -130,27 +134,19 @@ PWinObj::move(int x, int y, bool do_virtual)
     XMoveWindow(_dpy, _window, _gm.x, _gm.y);
 }
 
-//! @brief Interface method for virtual desktop placing
-void
-PWinObj::moveVirtual(int x, int y)
-{
-    _v_x = x;
-    _v_y = y;
-}
-
 //! @brief Resizes the window and updates _gm.
 void
 PWinObj::resize(uint width, uint height)
 {
+    if (! width || ! height) {
 #ifdef DEBUG
-    if ((width == 0) || (height == 0)) {
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "PWinObj(" << this << ")::resize(" << width << "," << height
              << ")" << endl << " *** invalid geometry, _window = "
              << _window << endl;
+#endif // DEBUG
         return;
     }
-#endif // DEBUG
 
     _gm.width = width;
     _gm.height = height;
@@ -158,6 +154,32 @@ PWinObj::resize(uint width, uint height)
     XResizeWindow(_dpy, _window, _gm.width, _gm.height);
 }
 
+//! @brief Move and resize window in one call.
+//! @param x X position.
+//! @param y Y Position.
+//! @param width Width.
+//! @param height Height.
+void
+PWinObj::moveResize(int x, int y, uint width, uint height)
+{
+    if (! width || ! height) {
+#ifdef DEBUG
+        cerr << __FILE__ << "@" << __LINE__ << ": "
+             << "PWinObj(" << this << ")::moveResize(" << width << "," << height
+             << ")" << endl << " *** invalid geometry, _window = "
+             << _window << endl;
+#endif // DEBUG
+        return;
+    }
+
+    _gm.x = x;
+    _gm.y = y;
+    _gm.width = width;
+    _gm.height = height;
+
+    XMoveResizeWindow(_dpy, _window, _gm.x, _gm.y, _gm.width, _gm.height);
+}
+
 //! @brief Only sets _workspace to workspace.
 void
 PWinObj::setWorkspace(uint workspace)
@@ -194,22 +216,20 @@ PWinObj::setHidden(bool hidden)
 }
 
 //! @brief Executes XSetInputFocus on the appropriate window.
-bool
+void
 PWinObj::giveInputFocus(void)
 {
-    if ((_mapped == false) || (_focusable == false)) {
+    if (! _mapped  || ! _focusable) {
 #ifdef DEBUG
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "PWinObj(" << this << ")::giveInputFocus()" << endl
              << " *** non focusable window."
              << " mapped: " << _mapped  << " focusable: " << _focusable << endl;
 #endif // DEBUG
-        return false;
+        return;
     }
 
     XSetInputFocus(_dpy, _window, RevertToPointerRoot, CurrentTime);
-
-    return true;
 }
 
 //! @brief Reparents and sets _parent member
@@ -220,6 +240,15 @@ PWinObj::reparent(PWinObj *wo, int x, int y)
     XReparentWindow(_dpy, _window, wo->getWindow(), x, y);
 }
 
+//! @brief Get required size to hold content for window
+//! @param request Geometry filled with size request.
+//! @return true if geometry is filled in, else false
+bool
+PWinObj::getSizeRequest(Geometry &request)
+{
+    return false;
+}
+
 //! @brief Adds PWinObj to _wo_list.
 void
 PWinObj::woListAdd(PWinObj *wo)
