@@ -26,17 +26,17 @@ using std::endl;
 
 using std::map;
 
-ScreenResources *ScreenResources::_instance = NULL;
+ScreenResources *ScreenResources::_instance = 0;
 
 //! @brief ScreenResources constructor
-ScreenResources::ScreenResources(void) :
-        _pixmap_handler(NULL)
+ScreenResources::ScreenResources(void)
+    : _pixmap_handler(0)
 {
-    if (_instance != NULL) {
+    if (_instance) {
 #ifdef DEBUG
         cerr << __FILE__ << "@" << __LINE__ << ": "
              << "ScreenResources(" << this << ")::ScreenResources()" << endl
-             << " *** _instance allready set: " << _instance << endl;
+             << " *** _instance already set: " << _instance << endl;
 #endif // DEBUG
     }
     _instance = this;
@@ -71,9 +71,9 @@ ScreenResources::~ScreenResources(void)
         XFreeCursor(PScreen::instance()->getDpy(), c_it->second);
     }
 
-    if (_pixmap_handler != NULL) {
+    if (_pixmap_handler) {
         delete _pixmap_handler;
     }
 
-    _instance = NULL;
+    _instance = 0;
 }
