@@ -1,6 +1,6 @@
 //
 // main.cc for pekwm
-// Copyright (C) 2003-2004 Claes Nasten <pekdon{@}pekdon{.}net>
+// Copyright © 2003-2008 Claes Nästén <me@pekdon.net>
 //
 // main.cc for aewm++
 // Copyright (C) 2000 Frank Hale <frankhale@yahoo.com>
@@ -10,10 +10,14 @@
 // See the LICENSE file for more information.
 //
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
+
 #include "PWinObj.hh"
 #include "PDecor.hh"
 #include "Client.hh"
+#include "Compat.hh"
 #include "Frame.hh"
 #include "WindowManager.hh"
 #include "Util.hh"
@@ -70,6 +74,7 @@ main(int argc, char **argv)
     string command_line;
 
     setlocale(LC_CTYPE, "");
+    Util::iconv_init();
 
     // build commandline
     for (int i = 0; i < argc; ++i) {
@@ -80,7 +85,7 @@ main(int argc, char **argv)
     for (int i = 1; i < argc; ++i)	{
         if ((strcmp("--display", argv[i]) == 0) && ((i + 1) < argc)) {
             setenv("DISPLAY", argv[++i], 1);
-        }	else if ((strcmp("--config", argv[i]) == 0) && ((i + 1) < argc)) {
+        } else if ((strcmp("--config", argv[i]) == 0) && ((i + 1) < argc)) {
             config_file = argv[++i];
         } else if (strcmp("--version", argv[i]) == 0) {
             Info::printVersion();
@@ -88,15 +93,20 @@ main(int argc, char **argv)
         } else if (strcmp("--info", argv[i]) == 0) {
             Info::printInfo();
             exit(0);
-        } else if (strcmp("--help", argv[i]) || !strcmp("-h", argv[i]) == 0) {
+        } else if (strcmp("--help", argv[i]) || ! strcmp("-h", argv[i]) == 0) {
             Info::printUsage();
             exit(0);
         }
     }
 
-    // read in the environment
-    if ((config_file.size() == 0) && (getenv("PEKWM_CONFIG_FILE") != NULL)) {
-        config_file = getenv("PEKWM_CONFIG_FILE");
+    // Get configuration file if none was specified as a parameter,
+    // default to reading environment, if not set get ~/.pekwm/config
+    if (config_file.size() == 0) {
+        if (getenv("PEKWM_CONFIG_FILE") && strlen(getenv("PEKWM_CONFIG_FILE"))) {
+            config_file = getenv("PEKWM_CONFIG_FILE");
+        } else {
+            config_file = string(getenv("HOME")) + string("/.pekwm/config");
+        }
     }
 
 #ifdef DEBUG
@@ -104,19 +114,21 @@ main(int argc, char **argv)
     Info::printInfo();
 #endif // DEBUG
 
-    WindowManager *pekwm = new WindowManager(command_line, config_file);
+    WindowManager::start(command_line, config_file);
 
     // see if we wanted to restart
-    if (pekwm->getRestartCommand().size() > 0) {
-        string restart_command = pekwm->getRestartCommand();
+    if (WindowManager::instance()->getRestartCommand().size() > 0) {
+        string restart_command = WindowManager::instance()->getRestartCommand();
 
         // cleanup before restarting
-        delete pekwm;
+        WindowManager::destroy();
+        Util::iconv_deinit();
 
-        execlp("/bin/sh", "sh" , "-c", restart_command.c_str(), (char*) NULL);
+        execlp("/bin/sh", "sh" , "-c", restart_command.c_str(), (char*) 0);
     }
 
-    delete pekwm;
+    WindowManager::destroy();
+    Util::iconv_deinit();
 
     return 0;
 }
